@@ -13,7 +13,7 @@ from itertools import repeat, chain
 from pathlib import Path
 from queue import Queue
 from threading import Thread, Lock, current_thread
-from typing import Optional, Dict, List, Any, cast, Union, Iterable, DefaultDict, Tuple
+from typing import Optional, Dict, List, Any, cast, Union, Iterable, DefaultDict, Tuple, Set
 from urllib.parse import urlparse
 
 from requests import Session, Response
@@ -415,7 +415,7 @@ class MediaContainer:
 
     def string_dump(self) -> str:
         return f"Name: {sanitize_name(self._name, False)!r}\nCourse: {self.course!r}\nSize: {HumanBytes.format_str(self.size)}\n" \
-               f"Time: {datetime.fromtimestamp(self.time).strftime(datetime_str)}\nIs downloaded: {self.checksum is not None}\n"
+               f"Time: {datetime.fromtimestamp(self.time).strftime(datetime_str)}\nIs downloaded: {self.checksum is not None}\nPath: {self.path}\n"
 
     def __str__(self) -> str:
         if config.absolute_path_filename:
@@ -461,7 +461,7 @@ class MediaContainer:
 
         self.current_size = 0
 
-        if not self.should_download:
+        if not self.should_download and self.url != 'https://www.eecs.tu-berlin.de/fileadmin/f4/fkIVdokumente/studium/Plagiate/Merkblatt_Plagiate_Fak.IV_05-2020.pdf':
             self.current_size = self.size
             self._done = True
             return
@@ -581,33 +581,6 @@ class Course:
                 for item in MediaType.list_dirs():
                     os.makedirs(self.path(item), exist_ok=True)
 
-    def download_videos(self, s: SessionWithKey) -> List[PreMediaContainer]:
-        if config.download_videos is False:
-            return []
-
-        url = "https://isis.tu-berlin.de/lib/ajax/service.php"
-        # Thank you isia-tub for this data <3
-        video_data = [{
-            "methodname": "mod_videoservice_get_videos",
-            "args": {"courseid": self.course_id}
-        }]
-
-        videos_res = s.get_(url, params={"sesskey": s.key}, json=video_data)
-
-        if videos_res is None or not videos_res.ok:
-            return []
-
-        videos_json = videos_res.json()[0]
-
-        if videos_json["error"]:
-            return []
-
-        videos_json = videos_json["data"]["videos"]
-        video_urls = [item["url"] for item in videos_json]
-        video_names = [item["title"].strip() + item["fileext"] for item in videos_json]
-
-        return [PreMediaContainer(url, self, MediaType.video, name) for url, name in zip(video_urls, video_names)]
-
     def download_documents(self, helper: RequestHelper) -> List[PreMediaContainer]:
         content = helper.post_REST("core_course_get_contents", {"courseid": self.course_id})
         if content is None or isinstance(content, dict) and "exception" in content:
@@ -714,6 +687,7 @@ class RequestHelper:
     course_id_mapping: Dict[int, Course] = {}
     _instance: Optional[RequestHelper] = None
     _instance_init: bool = False
+    _lock = Lock()
 
     def __init__(self, user: User, status: Optional[RequestHelperStatus] = None):
         if self._instance_init:
@@ -796,23 +770,23 @@ class RequestHelper:
         return response.json()
 
     def download_content(self, status: Optional[RequestHelperStatus] = None) -> Dict[MediaType, List[MediaContainer]]:
-        exception_lock = Lock()
-
         if status is not None:
             status.set_total(len(self.courses))
 
         if enable_multithread:
             with ThreadPoolExecutor(discover_num_threads) as ex:
+                # Note the use of .map() instead of .submit(). This is done so in both cases the variable can be of type `Iterable`.
+                # If done with .submit() one would have to wrap the function call into a `Future` which is too cumbersome.
                 _mod_assign = ex.map(self._download_mod_assign, [0])
-                _document_containers = ex.map(self._download_documents, self.courses, repeat(exception_lock))
-                _video_containers = ex.map(self._download_videos, self.courses, repeat(exception_lock), repeat(status))
+                _video_containers = ex.map(self._download_videos, [0])
+                _document_containers = ex.map(self._download_documents, self.courses, repeat(status))
 
         else:
-            _mod_assign = iter([self._download_mod_assign()])
-            _document_containers = iter([self._download_documents(course, exception_lock) for course in self.courses])
-            _video_containers = iter([self._download_videos(course, exception_lock, status) for course in self.courses])
+            _mod_assign = iter([self._download_mod_assign(0)])
+            _video_containers = iter([self._download_videos(0)])
+            _document_containers = iter([self._download_documents(course, status) for course in self.courses])
 
-        pre_containers = [item for row in chain(_document_containers, _video_containers, _mod_assign) for item in row]
+        pre_containers = [item for row in filter(lambda x: x is not None, chain(_document_containers, _video_containers, _mod_assign)) for item in row]
         pre_containers = list({f"{item.course} {item.url}": item for item in pre_containers}.values())
         random.shuffle(pre_containers)
 
@@ -843,41 +817,76 @@ class RequestHelper:
         return {typ: sorted(item, key=lambda x: x.time, reverse=True) for typ, item in mapping.items()}
 
     def _download_mod_assign(self, _: Any = None) -> List[PreMediaContainer]:
-        all_content = []
-        _assignments = self.post_REST("mod_assign_get_assignments", use_timeout=False)
-        if _assignments is None:
-            return []
+        try:
+            all_content = []
+            _assignments = self.post_REST("mod_assign_get_assignments", use_timeout=False)
+            if _assignments is None:
+                assignments = cast(Dict[str, Any], _assignments)
 
-        assignments = cast(Dict[str, Any], _assignments)
+                allowed_ids = {item.course_id for item in self.courses}
+                for _course in assignments["courses"]:
+                    if _course["id"] in allowed_ids:
+                        for assignment in _course["assignments"]:
+                            if "introattachments" not in assignment:
+                                continue
 
-        allowed_ids = {item.course_id for item in self.courses}
-        for _course in assignments["courses"]:
-            if _course["id"] in allowed_ids:
-                for assignment in _course["assignments"]:
-                    if "introattachments" not in assignment:
-                        continue
+                            for file in assignment["introattachments"]:
+                                file["filepath"] = assignment["name"]
+                                all_content.append(PreMediaContainer(
+                                    file["fileurl"], RequestHelper.course_id_mapping[_course["id"]], MediaType.document,
+                                    file["filename"], file["filepath"], file["filesize"], file["timemodified"])
+                                )
 
-                    for file in assignment["introattachments"]:
-                        file["filepath"] = assignment["name"]
-                        all_content.append(PreMediaContainer(file["fileurl"], RequestHelper.course_id_mapping[_course["id"]], MediaType.document,
-                                                             file["filename"], file["filepath"], file["filesize"], file["timemodified"]))
+                return all_content
 
-        return all_content
+        except Exception as ex:
+            with self._lock:
+                generate_error_message(ex)
 
-    def _download_documents(self, course: Course, exception_lock: Lock) -> List[PreMediaContainer]:
+        return []
+
+    def _download_videos(self, _: Any) -> List[PreMediaContainer]:
+        try:
+            if config.download_videos is False:
+                return []
+
+            url = "https://isis.tu-berlin.de/lib/ajax/service.php"
+            # Thank you isia-tub for discovering this service.
+            video_data = [{
+                "methodname": "mod_videoservice_get_videos",
+                "args": {"courseid": course.course_id},
+                "index": i
+            } for i, course in enumerate(self.courses)]
+
+            videos_res = self.session.get_(url, params={"sesskey": self.session.key}, json=video_data)
+
+            if videos_res is None or not videos_res.ok:
+                return []
+
+            res = []
+            for course, video in zip(self.courses, videos_res.json()):
+                if video["error"]:
+                    continue
+                assert course.course_id == video["data"]["courseid"]
+
+                videos_json = video["data"]["videos"]
+                video_urls = [item["url"] for item in videos_json]
+                video_names = [item["title"].strip() + item["fileext"] for item in videos_json]
+
+                res.extend([PreMediaContainer(url, course, MediaType.video, name) for url, name in zip(video_urls, video_names)])
+
+            return res
+
+        except Exception as ex:
+            with self._lock:
+                generate_error_message(ex)
+
+    def _download_documents(self, course: Course, status: Optional[RequestHelperStatus] = None) -> List[PreMediaContainer]:
         try:
             return course.download_documents(self)
 
         except Exception as ex:
-            with exception_lock:
-                generate_error_message(ex)
-
-    def _download_videos(self, course: Course, exception_lock: Lock, status: Optional[RequestHelperStatus] = None) -> List[PreMediaContainer]:
-        try:
-            return course.download_videos(self.session)
-
-        except Exception as ex:
-            with exception_lock:
+            with self._lock:
                 generate_error_message(ex)
 
         finally:
@@ -898,22 +907,8 @@ def check_for_conflicts_in_files(files: List[MediaContainer]) -> List[MediaConta
 
     files = new_files
 
-    # First check for the same urls across the entire list
-    hard_link_conflicts: DefaultDict[str, List[MediaContainer]] = defaultdict(list)
-
-    for file in {file.path: file for file in files}.values():
-        hard_link_conflicts[file.download_url].append(file)
-
-    for conflict in hard_link_conflicts.values():
-        if len(conflict) != 1:
-            conflict.sort(key=lambda x: x.time)
-            conflict[0]._links.extend(conflict[1:])
-            final_list.append(conflict[0])
-            for item in conflict[0]._links:
-                files.remove(item)
-
     # Now check for files with the same size in each course
-    hard_link_conflicts = defaultdict(list)
+    hard_link_conflicts: DefaultDict[str, List[MediaContainer]] = defaultdict(list)
 
     for file in {file.path: file for file in files}.values():
         hard_link_conflicts[f"{file.course.course_id} {file._name} {file.size}"].append(file)
@@ -953,7 +948,29 @@ def check_for_conflicts_in_files(files: List[MediaContainer]) -> List[MediaConta
             logger.assert_fail(f"conflict: {[{x: getattr(item, x) for x in item.__slots__} for item in conflict]}")
             continue
 
-    return final_list
+    # TODO: This takes too long.
+
+    # Finally filter the remaining files based on the url
+    hard_link_conflicts = defaultdict(list)
+
+    for file in {file.path: file for file in files}.values():
+        hard_link_conflicts[file.download_url].append(file)
+
+    conflict_urls: Set[str] = set()
+    new_files = []
+
+    for conflict in hard_link_conflicts.values():
+        if len(conflict) != 1:
+            conflict.sort(key=lambda x: x.time)
+            conflict[0]._links.extend(conflict[1:])
+            conflict_urls.add(conflict[0].url)
+            new_files.append(conflict[0])
+
+            final_list.append(conflict[0])
+            for item in conflict[0]._links:
+                final_list.remove(item)
+
+    return [file for file in final_list if file.url not in conflict_urls] + new_files
 
 
 class Downloader(Thread):
